@@ -4,17 +4,16 @@
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  the Free Software Foundation; version 2 of the License ONLY.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -57,8 +56,8 @@
 #endif
 #endif
 
-#include "xfconf-gvaluefuncs.h"
-#include "xfconf-common-private.h"
+#include "common/xfconf-gvaluefuncs.h"
+#include "common/xfconf-common-private.h"
 #include "xfconf/xfconf.h"
 
 static gboolean version = FALSE;
@@ -69,68 +68,39 @@ static gboolean reset = FALSE;
 static gboolean recursive = FALSE;
 static gboolean force_array = FALSE;
 static gboolean monitor = FALSE;
+static gboolean toggle = FALSE;
 static gchar *channel_name = NULL;
 static gchar *property_name = NULL;
 static gchar **set_value = NULL;
 static gchar **type = NULL;
-static gchar *import_file = NULL;
-static gchar *export_file = NULL;
 
 static void
 xfconf_query_monitor (XfconfChannel *channel, const gchar *changed_property, GValue *property_value)
 {
     gchar *string;
-    
-    if (property_name && !g_str_has_prefix (changed_property, property_name))
+
+    if(property_name && !g_str_has_prefix(changed_property, property_name))
         return;
 
-    if (property_value && G_IS_VALUE (property_value))
+    if(property_value && G_IS_VALUE (property_value))
     {
-        if (verbose)
+        if(verbose)
         {
-           string = _xfconf_string_from_gvalue (property_value);
-           g_print (_("Property '%s' changed: %s\n"), changed_property, string);
-           g_free (string);
+             string = _xfconf_string_from_gvalue(property_value);
+             g_print("%s: %s (%s)\n", _("set"), changed_property, string);
+             g_free(string);
        }
        else
        {
-           g_print (_("Property '%s' changed\n"), changed_property);
+             g_print("%s: %s\n", _("set"), changed_property);
        }
     }
     else
     {
-        g_print (_("Property '%s' removed\n"), changed_property);
+        g_print("%s: %s\n", _("reset"), changed_property);
     }
 }
 
-static gboolean
-xfconf_query_import_channel (XfconfChannel *channel, gint fd, GError **error)
-{
-    if (error != NULL)
-    {
-        g_set_error (error, G_FILE_ERROR, 1, _("Export method not yet implemented"));
-    }
-    else
-    {
-        g_warning ("--import: Method not implemented");
-    }
-    return FALSE;
-}
-
-static gboolean
-xfconf_query_export_channel (XfconfChannel *channel, gint fd, GError **error)
-{
-    if (error != NULL)
-    {
-        g_set_error (error, G_FILE_ERROR, 1, _("Export not yet implemented"));
-    }
-    else
-    {
-        g_warning ("--export: Method not implemented");
-    }
-    return FALSE;
-}
-
 static void
 xfconf_query_get_propname_size (gpointer key, gpointer value, gpointer user_data)
 {
@@ -146,7 +116,7 @@ static void
 xfconf_query_list_sorted (gpointer key, gpointer value, gpointer user_data)
 {
   GSList **listp = user_data;
-  
+
   *listp = g_slist_insert_sorted (*listp, key, (GCompareFunc) g_utf8_collate);
 }
 
@@ -157,7 +127,7 @@ xfconf_query_list_contents (GSList *sorted_contents, GHashTable *channel_content
     gchar *format = verbose ? g_strdup_printf ("%%-%ds%%s\n", size + 2) : NULL;
     GValue *property_value;
     gchar *string;
-    
+
     for (li = sorted_contents; li != NULL; li = li->next)
     {
         if (verbose)
@@ -181,10 +151,25 @@ xfconf_query_list_contents (GSList *sorted_contents, GHashTable *channel_content
             g_print ("%s\n", (gchar *) li->data);
         }
     }
-    
+
     g_free (format);
 }
 
+static void
+xfconf_query_printerr(const gchar *message,
+                      ...)
+{
+    va_list args;
+    gchar *str;
+
+    va_start(args, message);
+    str = g_strdup_vprintf(message, args);
+    va_end(args);
+
+    g_printerr("%s.\n", str);
+    g_free(str);
+}
+
 static GOptionEntry entries[] =
 {
      {   "version", 'V', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &version,
@@ -231,16 +216,10 @@ static GOptionEntry entries[] =
         N_("Force array even if only one element"),
         NULL
     },
-/*
-    {   "export", 'x', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING, &export_file,
-        N_("Export channel to file"),
-        NULL,
+    {   "toggle", 'T', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &toggle,
+        N_("Invert an existing boolean property"),
+        NULL
     },
-    {   "import", 'i', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING, &import_file,
-        N_("Import channel from file"),
-        NULL,
-    },
-*/
     {   "monitor", 'm', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &monitor,
         N_("Monitor a channel for property changes"),
         NULL,
@@ -248,14 +227,12 @@ static GOptionEntry entries[] =
     { NULL }
 };
 
-int 
+int
 main(int argc, char **argv)
 {
-    GError *cli_error = NULL;
     GError *error= NULL;
     XfconfChannel *channel = NULL;
     gboolean prop_exists;
-    gint fd = -1;
     GOptionContext *context;
 
 #ifdef ENABLE_NLS
@@ -266,25 +243,31 @@ main(int argc, char **argv)
     g_type_init();
     if(!xfconf_init(&error))
     {
-        g_critical("Failed to init libxfconf: %s\n", error->message);
+        xfconf_query_printerr(_("Failed to init libxfconf: %s"), error->message);
         g_error_free(error);
-        return 1;
+        return EXIT_FAILURE;
     }
 
-    context = g_option_context_new("- xfconf commandline utility");
-
+    context = g_option_context_new(_("- Xfconf commandline utility"));
     g_option_context_add_main_entries(context, entries, GETTEXT_PACKAGE);
 
-    if(!g_option_context_parse(context, &argc, &argv, &cli_error))
+    if(!g_option_context_parse(context, &argc, &argv, &error))
     {
-        g_printerr("option parsing failed: %s\n", cli_error->message);
-        return 1;
+        xfconf_query_printerr(_("Option parsing failed: %s"), error->message);
+        g_error_free(error);
+        return EXIT_FAILURE;
     }
 
     if(version)
     {
-        g_print("xfconf-query %s\n", PACKAGE_VERSION);
-        return 0;
+        g_print("xfconf-query");
+        g_print(" %s\n\n", PACKAGE_VERSION);
+        g_print("%s\n", "Copyright (c) 2008-2011");
+        g_print("\t%s\n\n", _("The Xfce development team. All rights reserved."));
+        g_print(_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
+        g_print("\n");
+
+        return EXIT_SUCCESS;
     }
 
     /** Check if the channel is specified */
@@ -293,7 +276,7 @@ main(int argc, char **argv)
         gchar **channels;
         gint i;
 
-        g_print("Channels:\n");
+        g_print("%s\n", _("Channels:"));
 
         channels = xfconf_list_channels();
         if(G_LIKELY(channels)) {
@@ -301,57 +284,44 @@ main(int argc, char **argv)
                 g_print("  %s\n", channels[i]);
             g_strfreev(channels);
         } else
-            return 1;
+            return EXIT_FAILURE;
 
-        return 0;
+        return EXIT_SUCCESS;
     }
 
     /** Check if the property is specified */
-    if(!property_name && !list && !export_file && !import_file && !monitor)
+    if(!property_name && !list && !monitor)
     {
-        g_printerr("No property specified, aborting...\n");
-        return 1;
+        xfconf_query_printerr(_("No property specified"));
+        return EXIT_FAILURE;
     }
 
     if (create && reset)
     {
-        g_printerr("--create and --reset options can not be used together,\naborting...\n");
-        return 1;
+        xfconf_query_printerr(_("--create and --reset options can not be used together"));
+        return EXIT_FAILURE;
     }
 
     if ((create || reset) && (list))
     {
-        g_printerr("--create and --reset options can not be used together with\n --list\naborting...\n");
-        return 1;
-    }
-
-    if (import_file && export_file)
-    {
-        g_printerr("--import and --export options can not be used together,\naborting...\n");
-        return 1;
-    }
-
-    if ((import_file || export_file) && (list || property_name || create || reset))
-    {
-        g_printerr("--import and --export options can not be used together with\n --create, --reset, --property and --list,\naborting...\n");
-        return 1;
+        xfconf_query_printerr(_("--create and --reset options can not be used together with --list"));
+        return EXIT_FAILURE;
     }
 
     channel = xfconf_channel_new(channel_name);
-    
+
     if (monitor)
     {
         GMainLoop *loop;
 
         g_signal_connect (G_OBJECT (channel), "property-changed", G_CALLBACK (xfconf_query_monitor), NULL);
-        
-        g_print ("\n");
-        g_print (_("Start monitoring channel '%s':"), channel_name);
-        g_print ("\n------------------------------------------------\n\n");
-        
+
+        g_print (_("Start monitoring channel \"%s\":"), channel_name);
+        g_print ("\n\n");
+
         loop = g_main_loop_new (NULL, TRUE);
         g_main_loop_run (loop);
-        g_main_loop_unref (loop);        
+        g_main_loop_unref (loop);
     }
 
     if (property_name && !list)
@@ -368,11 +338,28 @@ main(int argc, char **argv)
 
             if(!xfconf_channel_get_property(channel, property_name, &value))
             {
-                g_printerr(_("Property \"%s\" does not exist on channel \"%s\".\n"),
-                           property_name, channel_name);
-                return 1;
+                xfconf_query_printerr(_("Property \"%s\" does not exist on channel \"%s\""),
+                                      property_name, channel_name);
+                return EXIT_FAILURE;
             }
-            
+
+            if (toggle)
+            {
+                if(G_VALUE_HOLDS_BOOLEAN(&value))
+                {
+                    if(xfconf_channel_set_bool(channel, property_name, !g_value_get_boolean(&value)))
+                        return EXIT_SUCCESS;
+                    else
+                        xfconf_query_printerr(_("Failed to set property"));
+                }
+                else
+                {
+                    xfconf_query_printerr(_("--toggle only works with boolean values"));
+                }
+
+                return EXIT_FAILURE;
+            }
+
             if(XFCONF_TYPE_G_VALUE_ARRAY != G_VALUE_TYPE(&value))
             {
                 gchar *str_val = _xfconf_string_from_gvalue(&value);
@@ -385,7 +372,8 @@ main(int argc, char **argv)
                 GPtrArray *arr = g_value_get_boxed(&value);
                 guint i;
 
-                g_print(_("Value is an array with %d items:\n\n"), arr->len);
+                g_print(_("Value is an array with %d items:"), arr->len);
+                g_print("\n\n");
 
                 for(i = 0; i < arr->len; ++i)
                 {
@@ -408,16 +396,16 @@ main(int argc, char **argv)
             prop_exists = xfconf_channel_has_property(channel, property_name);
             if(!prop_exists && !create)
             {
-                g_printerr(_("Property \"%s\" does not exist on channel \"%s\".  If a new\n"
-                             "property should be created, use the --create option.\n"),
-                             property_name, channel_name);
-                return 1;
+                xfconf_query_printerr(_("Property \"%s\" does not exist on channel \"%s\". If a new "
+                                      "property should be created, use the --create option"),
+                                      property_name, channel_name);
+                return EXIT_FAILURE;
             }
 
             if(!prop_exists && (!type || !type[0]))
             {
-                g_printerr(_("When creating a new property, the value type must be specified.\n"));
-                return 1;
+                xfconf_query_printerr(_("When creating a new property, the value type must be specified"));
+                return EXIT_FAILURE;
             }
 
             if(prop_exists && !(type && type[0]))
@@ -426,8 +414,8 @@ main(int argc, char **argv)
                  * didn't specify one */
                 if(!xfconf_channel_get_property(channel, property_name, &value))
                 {
-                    g_printerr(_("Failed to get the existing type for the value.\n"));
-                    return 1;
+                    xfconf_query_printerr(_("Failed to get the existing type for the value"));
+                    return EXIT_FAILURE;
                 }
             }
 
@@ -444,14 +432,14 @@ main(int argc, char **argv)
 
                 if(G_TYPE_INVALID == gtype || G_TYPE_NONE == gtype)
                 {
-                    g_printerr(_("Unable to determine the type of the value.\n"));
-                    return 1;
+                    xfconf_query_printerr(_("Unable to determine the type of the value"));
+                    return EXIT_FAILURE;
                 }
 
                 if(XFCONF_TYPE_G_VALUE_ARRAY == gtype)
                 {
-                    g_printerr(_("A value type must be specified to change an array into a single value.\n"));
-                    return 1;
+                    xfconf_query_printerr(_("A value type must be specified to change an array into a single value"));
+                    return EXIT_FAILURE;
                 }
 
                 if(G_VALUE_TYPE(&value))
@@ -460,15 +448,15 @@ main(int argc, char **argv)
                 g_value_init(&value, gtype);
                 if(!_xfconf_gvalue_from_string(&value, set_value[0]))
                 {
-                    g_printerr(_("Unable to convert \"%s\" to type \"%s\"\n"),
-                                   set_value[0], g_type_name(gtype));
-                    return 1;
+                    xfconf_query_printerr(_("Unable to convert \"%s\" to type \"%s\""),
+                                          set_value[0], g_type_name(gtype));
+                    return EXIT_FAILURE;
                 }
 
                 if(!xfconf_channel_set_property(channel, property_name, &value))
                 {
-                    g_printerr(_("Failed to set property.\n"));
-                    return 1;
+                    xfconf_query_printerr(_("Failed to set property"));
+                    return EXIT_FAILURE;
                 }
             }
             else
@@ -492,9 +480,9 @@ main(int argc, char **argv)
 
                 if(new_values != new_types)
                 {
-                    g_printerr(_("There are %d new values, but only %d types could be determined.\n"),
-                               new_values, new_types);
-                    return 1;
+                    xfconf_query_printerr(_("There are %d new values, but only %d types could be determined"),
+                                          new_values, new_types);
+                    return EXIT_FAILURE;
                 }
 
                 arr_new = g_ptr_array_sized_new(new_values);
@@ -513,18 +501,17 @@ main(int argc, char **argv)
 
                     if(G_TYPE_INVALID == gtype || G_TYPE_NONE == gtype)
                     {
-                        g_printerr(_("Unable to determine type of value at index %d.\n"),
-                                   i);
-                        return 1;
+                        xfconf_query_printerr(_("Unable to determine type of value at index %d"), i);
+                        return EXIT_FAILURE;
                     }
 
                     value_new = g_new0(GValue, 1);
                     g_value_init(value_new, gtype);
                     if(!_xfconf_gvalue_from_string(value_new, set_value[i]))
                     {
-                        g_printerr(_("Unable to convert \"%s\" to type \"%s\"\n"),
-                                   set_value[i], g_type_name(gtype));
-                        return 1;
+                        xfconf_query_printerr(_("Unable to convert \"%s\" to type \"%s\""),
+                                              set_value[i], g_type_name(gtype));
+                        return EXIT_FAILURE;
                     }
                     g_ptr_array_add(arr_new, value_new);
                 }
@@ -537,8 +524,8 @@ main(int argc, char **argv)
 
                 if(!xfconf_channel_set_property(channel, property_name, &value))
                 {
-                    g_printerr(_("Failed to set property.\n"));
-                    return 1;
+                    xfconf_query_printerr(_("Failed to set property"));
+                    return EXIT_FAILURE;
                 }
             }
         }
@@ -554,77 +541,20 @@ main(int argc, char **argv)
 
             if (verbose)
                 g_hash_table_foreach (channel_contents, (GHFunc)xfconf_query_get_propname_size, &size);
-            
+
             g_hash_table_foreach (channel_contents, (GHFunc)xfconf_query_list_sorted, &sorted_contents);
-            
+
             xfconf_query_list_contents (sorted_contents, channel_contents, size);
-            
+
             g_slist_free (sorted_contents);
             g_hash_table_destroy (channel_contents);
         }
         else
         {
-            g_print(_("Channel '%s' contains no properties\n"), channel_name);
+            g_print(_("Channel \"%s\" contains no properties"), channel_name);
+            g_print(".\n");
         }
     }
 
-    if (export_file)
-    {
-        if (!strcmp(export_file, "-"))
-        {
-            /* Use stdout */
-            fd = fileno (stdout);
-        }
-        else
-        {
-            fd = open (export_file, O_CREAT | O_EXCL | O_WRONLY, 0);
-            if (fd < 0)
-            {
-                g_printerr (_("Could not create export file \"%s\": %s\n"), export_file, strerror (errno));
-                return 1;
-            }
-        }
-
-        if (!xfconf_query_export_channel (channel, fd, &error))
-        {
-            if (fd != fileno (stdout))
-                close (fd);
-            g_printerr (_("Could not create export file \"%s\": %s\n"), export_file, error->message);
-            g_error_free (error);
-            return 1;
-        }
-        if (fd != fileno (stdout))
-            close (fd);
-    }
-
-    if (import_file)
-    {
-        if (!strcmp(import_file, "-"))
-        {
-            /* Use stdin */
-            fd = fileno (stdin);
-        }
-        else
-        {
-            fd = open (import_file, O_RDONLY, 0);
-            if (fd < 0)
-            {
-                g_printerr (_("Could not open import file \"%s\": %s\n"), import_file, strerror (errno));
-                return 1;
-            }
-        }
-
-        if (!xfconf_query_import_channel (channel, fd, &error))
-        {
-            if (fd != fileno (stdin))
-                close (fd);
-            g_printerr (_("Could not parse import file \"%s\": %s\n"), import_file, error->message);
-            g_error_free (error);
-            return 1;
-        }
-        if (fd != fileno (stdin))
-            close (fd);
-    }
-
-    return 0;
+    return EXIT_SUCCESS;
 }
