@@ -3,17 +3,18 @@
  *
  *  Copyright (c) 2009 Brian Tarricone <brian@tarricone.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; version 2 of the License ONLY.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; version 2
+ *  of the License ONLY.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
@@ -29,10 +30,10 @@
 #include "xfconf-channel.h"
 #include "xfconf-errors.h"
 #include "xfconf-dbus-bindings.h"
-#include "xfconf-gvaluefuncs.h"
+#include "common/xfconf-gvaluefuncs.h"
 #include "xfconf-private.h"
-#include "xfconf-marshal.h"
-#include "xfconf-common-private.h"
+#include "common/xfconf-marshal.h"
+#include "common/xfconf-common-private.h"
 #if 0
 #include "xfconf-types.h"
 #include "xfconf.h"
@@ -453,6 +454,13 @@ xfconf_cache_property_changed(DBusGProxy *proxy,
     if(strcmp(channel_name, cache->channel_name))
         return;
 
+    /* if a call was cancelled, we still receive a property-changed from
+     * that value, in that case, abort the emission of the signal. we can
+     * detect this because the new reply is not processed yet and thus
+     * there is still an old_prop in the hash table */
+    if(g_hash_table_lookup(cache->old_properties, property))
+        return;
+
     item = g_tree_lookup(cache->properties, property);
     if(item)
         changed = xfconf_cache_item_update(item, value);
@@ -743,10 +751,8 @@ xfconf_cache_set(XfconfCache *cache,
                 dbus_error_name = dbus_g_error_get_name(tmp_error);
             }
 
-            if(G_UNLIKELY(!dbus_error_name
-                          || strncmp(dbus_error_name, "org.xfce.Xfconf.Error.", 22)
-                          || (strcmp(dbus_error_name+22, "PropertyNotFound")
-                              && strcmp(dbus_error_name+22, "ChannelNotFound"))))
+            if(g_strcmp0(dbus_error_name, "org.xfce.Xfconf.Error.PropertyNotFound") != 0
+               && g_strcmp0(dbus_error_name, "org.xfce.Xfconf.Error.ChannelNotFound") != 0)
             {
                 /* this is bad... */
                 g_propagate_error(error, tmp_error);
