@@ -1018,10 +1018,11 @@ xfconf_channel_set_string(XfconfChannel *channel,
     GValue val = { 0, };
     gboolean ret;
 
-    g_return_val_if_fail(XFCONF_IS_CHANNEL(channel) && property && value, FALSE);
+    g_return_val_if_fail(XFCONF_IS_CHANNEL(channel) && property, FALSE);
+    g_return_val_if_fail(value == NULL || g_utf8_validate(value, -1, NULL), FALSE);
 
     g_value_init(&val, G_TYPE_STRING);
-    g_value_set_string(&val, value);
+    g_value_set_static_string(&val, value);
 
     ret = xfconf_channel_set_internal(channel, property, &val);
 
@@ -1320,7 +1321,12 @@ xfconf_channel_set_property(XfconfChannel *channel,
     GPtrArray *arr_new = NULL;
     gboolean ret;
 
-    g_return_val_if_fail(XFCONF_IS_CHANNEL(channel) && property && value,
+    g_return_val_if_fail(XFCONF_IS_CHANNEL(channel)
+                         && property
+                         && G_IS_VALUE(value), FALSE);
+    g_return_val_if_fail(!G_VALUE_HOLDS_STRING(value)
+                         || g_value_get_string(value) == NULL
+                         || g_utf8_validate(g_value_get_string(value), -1, NULL),
                          FALSE);
 
     /* intercept uint16/int16 since dbus-glib doesn't know how to send
@@ -1456,7 +1462,11 @@ xfconf_channel_get_array_valist(XfconfChannel *channel,
 
         switch(cur_value_type) {
             HANDLE_CASE(guchar, UCHAR, uchar)
+#if GLIB_CHECK_VERSION (2, 32, 0)
+            HANDLE_CASE(gchar, CHAR, schar)
+#else
             HANDLE_CASE(gchar, CHAR, char)
+#endif
             HANDLE_CASE(guint32, UINT, uint)
             HANDLE_CASE(gint32, INT, int)
             HANDLE_CASE(guint64, UINT64, uint64)
@@ -1632,7 +1642,11 @@ xfconf_channel_set_array_valist(XfconfChannel *channel,
 
         switch(cur_value_type) {
             HANDLE_CASE(guchar, UCHAR, uchar)
+#if GLIB_CHECK_VERSION (2, 32, 0)
+            HANDLE_CASE(gchar, CHAR, schar)
+#else
             HANDLE_CASE(gchar, CHAR, char)
+#endif
             HANDLE_CASE(guint32, UINT, uint)
             HANDLE_CASE(gint32, INT, int)
             HANDLE_CASE(guint64, UINT64, uint64)
@@ -1971,8 +1985,13 @@ xfconf_channel_get_structv(XfconfChannel *channel,
                 break;
 
             case G_TYPE_CHAR:
+#if GLIB_CHECK_VERSION (2, 32, 0)
+                SET_STRUCT_VAL(gchar, G_TYPE_CHAR, ALIGNOF_GCHAR,
+                               g_value_get_schar);
+#else
                 SET_STRUCT_VAL(gchar, G_TYPE_CHAR, ALIGNOF_GCHAR,
                                g_value_get_char);
+#endif
                 break;
 
             case G_TYPE_UINT:
@@ -2190,8 +2209,13 @@ xfconf_channel_set_structv(XfconfChannel *channel,
                 break;
 
             case G_TYPE_CHAR:
+#if GLIB_CHECK_VERSION (2, 32, 0)
+                GET_STRUCT_VAL(gchar, G_TYPE_CHAR, ALIGNOF_GCHAR,
+                               g_value_set_schar);
+#else
                 GET_STRUCT_VAL(gchar, G_TYPE_CHAR, ALIGNOF_GCHAR,
                                g_value_set_char);
+#endif
                 break;
 
             case G_TYPE_UINT:
