@@ -28,10 +28,10 @@
 
 static void xfconf_backend_base_init(gpointer g_class);
 
-static inline gboolean xfconf_property_is_valid(const gchar *property,
-                                                GError **error);
-static inline gboolean xfconf_channel_is_valid(const gchar *channel,
-                                               GError **error);
+static gboolean xfconf_property_is_valid(const gchar *property,
+                                         GError **error);
+static gboolean xfconf_channel_is_valid(const gchar *channel,
+                                        GError **error);
 
 /**
  * XfconfBackendInterface:
@@ -52,7 +52,7 @@ static inline gboolean xfconf_channel_is_valid(const gchar *channel,
 
 
 GType
-xfconf_backend_get_type()
+xfconf_backend_get_type(void)
 {
     static GType backend_type = 0;
     
@@ -91,7 +91,7 @@ xfconf_backend_base_init(gpointer g_class)
 
 
 
-static inline gboolean
+static gboolean
 xfconf_property_is_valid(const gchar *property,
                          GError **error)
 {
@@ -132,7 +132,7 @@ xfconf_property_is_valid(const gchar *property,
             if(error) {
                 g_set_error(error, XFCONF_ERROR,
                             XFCONF_ERROR_INVALID_PROPERTY,
-                            _("Property names cannot have two or more '/' characters in a row"));
+                            _("Property names cannot have two or more consecutive '/' characters"));
             }
             return FALSE;
         }
@@ -151,7 +151,7 @@ xfconf_property_is_valid(const gchar *property,
     return TRUE;
 }
 
-static inline gboolean
+static gboolean
 xfconf_channel_is_valid(const gchar *channel,
                         GError **error)
 {
@@ -395,7 +395,7 @@ xfconf_backend_reset(XfconfBackend *backend,
     if(!recursive && (!*property || (property[0] == '/' && !property[1]))) {
         if(error) {
             g_set_error(error, XFCONF_ERROR, XFCONF_ERROR_INVALID_PROPERTY,
-                        _("The property name can only be empty or \"/\" if a recursive reset was specified."));
+                        _("The property name can only be empty or \"/\" if a recursive reset was specified"));
         }
         return FALSE;
     }
